@@ -116,7 +116,8 @@ XenaFilter = function(x,
 ##' Query URL of Datasets before Downloading
 ##' @author Shixiang Wang <w_shixiang@163.com>
 ##' @param x a [XenaHub] object
-##' @return a `tibble` contains hosts, datasets and url
+##' @return a `data.frame` contains hosts, datasets and url
+##' @importFrom dplyr filter select pull rename mutate
 ##' @export
 ##' @examples
 ##' \donttest{
@@ -125,44 +126,56 @@ XenaFilter = function(x,
 ##' xe_query = XenaQuery(xe)
 ##' }
 XenaQuery = function(x) {
-    hostsName = hosts(x)
-    datasetsName = datasets(x)
-
     message("This will check url status, please be patient.")
-    if (length(hostsName) == 1) {
-        urls = file.path(hostsName, "download", datasetsName)
-        # if file and file.gz all exist, download file # see issue#2
-        dlink = ifelse(!sapply(urls, httr::http_error),
-                       urls, paste0(urls, ".gz"))
-        query = data.frame(
-            hosts = hostsName,
-            datasets = datasetsName,
-            url = dlink,
-            stringsAsFactors = FALSE
-        )
-    } else{
-        # identify relationship of hosts and datasets
-        query = data.frame(
-            hosts = "NA",
-            datasets = datasetsName,
-            url = "NA",
-            stringsAsFactors = FALSE
-        )
-        for (onehost in hostsName) {
-            xe = XenaHub(hosts = onehost)
-            dataset_list = datasets(xe)
-            query$hosts[query$datasets %in% dataset_list] = hosts(xe)
-        }
-        urls = paste0(query$hosts, "/download/", query$datasets)
-        # if file and file.gz all exist, download file # see issue#2
-        query$url = ifelse(!sapply(urls, httr::http_error),
-                           urls, paste0(urls, ".gz"))
-
-    }
+    datasetsName = datasets(x)
+    query = UCSCXenaTools::XenaData %>%
+        dplyr::filter(XenaDatasets %in% datasetsName) %>%
+        dplyr::rename(hosts = XenaHosts, datasets = XenaDatasets) %>%
+        dplyr::mutate(url = file.path(hosts, "download", datasets)) %>%
+        dplyr::mutate(url = ifelse(!sapply(url, httr::http_error),
+                                    url, paste0(url, ".gz"))) %>%
+        dplyr::select(hosts, datasets, url) %>%
+        as.data.frame()
 
     invisible(query)
 }
 
+##' Query ProbeMap URL of Datasets
+##'
+##' If dataset has no ProbeMap, it will be ignored.
+##'
+##' @author Shixiang Wang <w_shixiang@163.com>
+##' @param x a [XenaHub] object
+##' @return a `data.frame` contains hosts, datasets and url
+##' @importFrom dplyr filter select pull rename mutate
+##' @export
+##' @examples
+##' \donttest{
+##' xe = XenaGenerate(subset = XenaHostNames == "tcgaHub")
+##' hosts(xe)
+##' xe_query = XenaQueryProbeMap(xe)
+##' }
+XenaQueryProbeMap = function(x) {
+    message("Check ProbeMap urls of datasets.")
+    datasetsName = datasets(x)
+    query = UCSCXenaTools::XenaData %>%
+        dplyr::filter((XenaDatasets %in% datasetsName) & (!is.na(ProbeMap)))
+
+    if (nrow(query) == 0) {
+        invisible(data.frame(stringsAsFactors = FALSE))
+    } else {
+        query = query %>%
+            dplyr::rename(hosts = XenaHosts, datasets = ProbeMap) %>%
+            dplyr::mutate(url = file.path(hosts, "download", datasets)) %>%
+            dplyr::mutate(url = ifelse(!sapply(url, httr::http_error),
+                                       url, paste0(url, ".gz"))) %>%
+            dplyr::select(hosts, datasets, url) %>%
+            as.data.frame()
+
+        invisible(unique(query))
+    }
+}
+
 ##' Download Datasets from UCSC Xena Hubs
 ##'
 ##' Avaliable datasets list: <https://xenabrowser.net/datapages/>
@@ -170,6 +183,7 @@ XenaQuery = function(x) {
 ##' @author Shixiang Wang <w_shixiang@163.com>
 ##' @param xquery a tibble object generated by [XenaQuery] function.
 ##' @param destdir specify a location to store download data. Default is system temp directory.
+##' @param download_probeMap if `TRUE`, also download ProbeMap data, which used for id mapping.
 ##' @param trans_slash logical, default is `FALSE`. If `TRUE`, transform slash '/' in dataset id
 ##' to '__'. This option is for backwards compatibility.
 ##' @param force logical. if `TRUE`, force to download data no matter whether files exist.
@@ -178,6 +192,7 @@ XenaQuery = function(x) {
 ##' @return a `tibble`
 ##' @export
 ##' @importFrom utils download.file
+##' @importFrom dplyr filter
 ##' @examples
 ##' \donttest{
 ##' xe = XenaGenerate(subset = XenaHostNames == "tcgaHub")
@@ -188,10 +203,19 @@ XenaQuery = function(x) {
 
 XenaDownload = function(xquery,
                         destdir = tempdir(),
+                        download_probeMap = FALSE,
                         trans_slash = FALSE,
                         force = FALSE,
                         ...) {
-    stopifnot(is.data.frame(xquery), c("url") %in% names(xquery))
+    stopifnot(is.data.frame(xquery), c("url") %in% names(xquery), is.logical(download_probeMap))
+
+    if (download_probeMap) {
+        xquery_probe = UCSCXenaTools::XenaData %>%
+            dplyr::filter(XenaDatasets %in% xquery$datasets) %>%
+            XenaGenerate() %>%
+            XenaQueryProbeMap()
+        xquery = rbind(xquery, xquery_probe)
+    }
 
     if (trans_slash) {
         xquery$fileNames = gsub(pattern = "/",
@@ -592,3 +616,4 @@ XenaBrowse = function(x, type = c("dataset", "cohort"), multiple=FALSE) {
     invisible(NULL)
 }
 
+utils::globalVariables(c("XenaDatasets", "XenaHosts", "ProbeMap"))
